﻿//////////////////////////////////////////////
// BufferManager.h
//
//////////////////////////////////////////////

/// Defines / Macros -------------------------

#pragma once

/// Forward decl -----------------------------

namespace nkGraphics
{
	class Buffer ;
}

/// Includes ---------------------------------

// nkGraphics
#include "../Dll/DllDefines.h"

// nkCommon
#include <nilkinsCommon/Patterns/SingletonClass.h>

// nkMemory
#include <NilkinsMemory/Containers/StringView.h>

// Standards
#include <mutex>
#include <string>
#include <unordered_map>

/// Class ------------------------------------

namespace nkGraphics
{	
	class DLL_GRAPHICS_EXPORT BufferManager : public nkCommon::SingletonClass<BufferManager>
	{
		friend class nkCommon::SingletonClass<BufferManager> ;

		private :
		
			// Attributs
			// Liste des buffers en memoire
			std::unordered_map<std::string, Buffer*> _memory ;

			// Threading
			std::mutex _mutex ;

		private :

			// Constructeur
			BufferManager () ;
			
		public :

			// Destructeur
			~BufferManager () ;
		
			// Manipulation mémoire
			Buffer* createOrRetrieve (const nkMemory::StringView& name) ;
			Buffer* get (const nkMemory::StringView& name) ;
			Buffer* getByIndex (unsigned int index) ;
			void rename (const nkMemory::StringView& oldName, const nkMemory::StringView& newName) ;
			void erase (const nkMemory::StringView& name) ;
	} ;
}