﻿//////////////////////////////////////////////
// VertexComposition.h
//
//////////////////////////////////////////////

/// Defines / Macros -------------------------

#pragma once

/// Includes ---------------------------------

// nkGraphics
#include "../Dll/DllDefines.h"

#include "../Utilities/DxDefinesWrapper.h"
#include "../Utilities/InfiniteMask.h"

// nkMemory
#include <NilkinsMemory/Containers/String.h>
#include <NilkinsMemory/Containers/StringView.h>

/// Class ------------------------------------
	
namespace nkGraphics
{
	class DLL_GRAPHICS_EXPORT VertexComposition final
	{	
		private :

			// Attributes
			// As a bitmask, used for input layouts
			InfiniteByteMask _asMask ;

		public :

			// Attributes
			// Formats used for the data
			FORMAT _positionFormat ;
			FORMAT _colorFormat ;
			FORMAT _uvFormat ;
			FORMAT _normalFormat ;
			FORMAT _tangentFormat ;
			FORMAT _binormalFormat ;

			// Topology of meshing
			PRIMITIVE_TOPOLOGY _topology ;

			// Which data should be part of a vertex
			bool _position ;
			bool _color ;
			bool _uv ;
			bool _normal ;
			bool _tangent ;
			bool _binormal ;

		public :

			// Constructor, destructor
			VertexComposition () ;
			VertexComposition (bool position, bool color, bool uv, bool normal, bool tangent, bool binormal) ;
			~VertexComposition () ;

			// Getters
			unsigned int getAsInt () const ;
			nkMemory::String getAsString () const ;
			const InfiniteByteMask& getAsMask () const ;
			unsigned short getAttributeNumber () const ;
			unsigned short getTotalComponentNumber () const ;
			unsigned short getTotalByteSize () const ;
			unsigned short getVertexPerPrimitive () const ;
			unsigned short getComponentPerFormat (FORMAT format) const ;

			// Refresh bitmask
			void refreshBitMask () ;

			// Opérators
			bool operator== (const VertexComposition& other) const ;
			bool operator< (const VertexComposition& other) const ;
	} ;
}