﻿//////////////////////////////////////////////
// FunctionParameter.h
//
//////////////////////////////////////////////

/// Defines / Macros -------------------------

#pragma once

/// Forward decl -----------------------------

namespace nkScripts
{
	class UserType ;
}

/// Includes ---------------------------------

// nkScripts
#include "../../Dll/DllDefines.h"

#include "../ScriptObjects/ScriptObjectReference.h"
#include "../ScriptObjects/ScriptObjectView.h"

#include "FunctionParameterType.h"

// nkMemory
#include <NilkinsMemory/Containers/String.h>
#include <NilkinsMemory/Containers/StringView.h>

// Standards
#include <string>

/// Internals --------------------------------

namespace nkScripts
{
	struct UserTypeHolder
	{
		// Il nous faut le pointeur et le nom du user type
		nkMemory::String _userTypeName ;
		void* _userData = nullptr ;
		ScriptObjectView _objectRef ;
	} ;
}

/// Class ------------------------------------

namespace nkScripts
{
	struct DLL_SCRIPTS_EXPORT FunctionParameter final
	{
		// Attributes
		union
		{
			bool _valBool ;
			int _valInt ;
			float _valFloat ;
			double _valDouble ;
			nkMemory::String _valString ;
			UserTypeHolder _valUser ;
			ScriptObjectReference _valScriptObject ;
		} ;

		FUNCTION_PARAMETER_TYPE _type ;

		// Functions
		// Constructeur, destructeur
		FunctionParameter () ;
		explicit FunctionParameter (bool val) ;
		explicit FunctionParameter (int val) ;
		explicit FunctionParameter (float val) ;
		explicit FunctionParameter (double val) ;
		FunctionParameter (const char* val) ;
		FunctionParameter (const nkMemory::StringView& val) ;
		FunctionParameter (void* val, const nkMemory::StringView& userTypeName, bool scriptOwner) ;
		FunctionParameter (const ScriptObjectReference& scriptObjectReference) ;
		FunctionParameter (const FunctionParameter& other) ;
		~FunctionParameter () ;

		// Helpers
		void prepareForInternalType () ;
		void prepareForString () ;
		void prepareForUserType () ;
		void prepareForScriptObjectReference () ;

		// Copie
		FunctionParameter& operator= (const FunctionParameter& other) ;

		// Statique retour pour void
		static FunctionParameter VOID ;
	} ;
}