﻿//////////////////////////////////////////////
// TreeView.h
//
//////////////////////////////////////////////

/// Defines / macro --------------------------

#pragma once

/// Includes ---------------------------------

// nkWinUi
#include "Component.h"

#include "../Dll/DllDefines.h"

#include "../Inputs/CallbackRepository.h"

// Standards
#include <map>
#include <string>
#include <vector>

/// Class ------------------------------------
	
namespace nkWinUi
{
	class DLL_WINUI_EXPORT TreeView : public Component
	{
		public :

			// Un item
			class DLL_WINUI_EXPORT Item
			{
				protected :
						
					// Attributs
					// Les fils
					std::vector<Item*> _children ;

					// Son texte
					std::string _text ;

					// Treeview père
					TreeView* _parentTree ;
					// Item père
					Item* _parentItem ;
					Item* _precedingItem ;

					// Les donnees
					void* _attachedData ;

				public :

					// Constructeur, destructeur
					Item () ;
					virtual ~Item () ;

					// Pour les fils
					Item* addChild (Item* child) ;
					Item* getChild (unsigned int index) const ;
					void disavowChild (Item* child) ;

					// Le texte
					void setText (const nkMemory::StringView& text) ;
					nkMemory::StringView getText () const ;

					// Les parents
					void setParentTree (TreeView* parent) ;
					TreeView* getParentTree () const ;
					void setParentItem (Item* item) ;
					Item* getParentItem () const ;
					void setPrecedingItem (Item* item) ;
					Item* getPrecedingItem () const ;

					// Les donnees
					void setAttachedData (void* data) ;
					void* getAttachedData () const ;

					// Export
					void exportIntrospection (nkExport::Node* rootNode) ;
			} ;

		protected :

			// Functions
			virtual Item* _allocateItem () const = 0 ;

		protected :

			// Attributes
			// Liste des items, accessibles via leurs donnees
			std::map<void*, Item*> _itemsFromData ;

			// Callbacks
			treeSelectionCallback _selectionCallback ;
			contextMenuCreationCallback _contextMenuCallback ;

			// Petits témoins de fonctionnement
			bool _selectOnRightClick ;
		
		public :
		
			// Constructeur, destructeur
			TreeView () ;
			~TreeView () ;

			// Getters
			Item* getItemFromData (void* data) const ;
			virtual Item* getSelectedItem () = 0 ;
			virtual Item* getItemAtAbsoluteCursor (const nkMaths::Point& absCoords) = 0 ;
			virtual Item* getItemAtRelativeCursor (const nkMaths::Point& relCoords) = 0 ;
			bool getSelectOnRightClick () const ;
			treeSelectionCallback getSelectionCallback () const ;

			// Setters
			void setSelectionOnRightClick (bool value) ;
			void setSelectionCallback (treeSelectionCallback value) ;
			void setContextMenuCallback (contextMenuCreationCallback value) ;

			// Gestion Items
			virtual Item* insertItem (const nkMemory::StringView& text, void* data, Item* parent = nullptr, Item* afterThisOne = nullptr) ;
			virtual void deleteItem (Item* toDelete, bool keepChildrenOnUpper = false) ;
			virtual void changeItemLabel (Item* toChange, const nkMemory::StringView& label) ;

			// Selection
			void selectItemFromItsData (void* data) ;
			virtual void selectItem (Item* item) = 0 ;
			void selectItemFromAbsoluteCursor (const nkMaths::Point& absCoords) ;
			void selectItemFromRelativeCursor (const nkMaths::Point& relCoords) ;

			// Import / Export
			virtual void exportIntrospection (nkExport::Node* rootNode) override ;
			virtual void importClassFromTree (nkExport::Node* rootNode) override ;
	} ;
}